/*#define DEBUG*/
/*
 * Copyright (C) 2002-4 Edscott Wilson Garcia
 * EMail: edscott@imp.mx
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/stat.h>

#include <sys/types.h>

#include <sys/wait.h>

#include <limits.h>

#include <memory.h>

#ifdef HAVE_STDARG_H
#include <stdarg.h>
#elif HAVE_VARARGS_H
#include <varargs.h>
#else
#error "no <stdarg.h> or <varargs.h>!"
#endif

#include <stdio.h>

#include <stdlib.h>

#include <string.h>

#include <time.h>

#include <unistd.h>

#include <regex.h>
#include <dirent.h>

#include <X11/Xlib.h>
#include <X11/Xproto.h>
#include <X11/Xatom.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include <dbh.h>

#include "glade_support.h"

#include "constants.h"
#include "types.h"

#include "basic_row.h"

#include "misc.h"
#include "modules.h"

#include "entry.h"
#include "icons.h"
#include "treeview.h"


extern root_t root[];

static gboolean diagnostics_disabled=FALSE;

    
static gchar *text_editors[]={
    "gvim",
    "emacs",
    "xedit",
    NULL
};
static gchar *text_types[]={
    "text/",
    "application/x-csh",
    "application/x-bsh",
    "application/x-ksh",
    "application/x-python",
    "application/x-shellscript",
    "application/x-tsh",
    "application/x-zsh",
    "application/xhtml+xml",
    "application/rtf",
    "application/x-cgi",
    "application/x-perl",
    "application/x-php",
    "application/x-theme",
    "application/x-trash",
    NULL
};


gchar **text_type_OK(const gchar *path)
{ 
  const gchar *t=MIME_get_type((const gchar *)path, TRUE);
  if (t) {
      int i;
      for (i=0;text_types[i];i++) if (strncmp(text_types[i],t,strlen(text_types[i]))==0){
	  return text_editors;
      }
  }
  return NULL;
}


/*
 * This function converts a time value specified in seconds since 1970-01-01
 * to a string representation. It shows the date and the time if the point
 * if less then six month in the past and only the date otherwise.
 * The exact format must be provided by a translation string. 
 *
 * The function should be thread-save since there are not used static
 * (or even global) variables if the system provided a localtime_r() function.
 *
 * Arguments:     when:    time in seconds since 1970-01-01
 *                string:  the result char-array in which the string is placed
 *                length:  the length of the string-array
 *                
 * Return value:  string on success, NULL on failure
 */
const gchar *
time_to_string (time_t when) 
{
      time_t          now             = time(NULL);
#ifdef HAVE_LOCALTIME_R
      struct tm       timestruct;
#endif
      struct tm*      timestruct_ptr;
      char	*formatstring;
      static gchar *s=NULL;
      gchar string[64];
      
#ifdef HAVE_MEMSET
      memset (string, 0, 64);
#else
      string[0]=0;
#endif
      
      formatstring = difftime(now, when) > 24*60*60*30*6
            /* strftime format for non-recent files (older than 6 months)  */
          ? _("%b %e  %Y")
            /* strftime format for recent files */
          : _("%b %e %H:%M");

#ifdef HAVE_LOCALTIME_R
      timestruct_ptr = &timestruct;
      localtime_r(&when, timestruct_ptr);
#else
      timestruct_ptr = localtime(&when);
#endif

      if (strftime(string, 64, formatstring, localtime(&when)) == 0) {
          return NULL;
      }
      g_free(s);
      s=g_strdup(my_utf_string(string));
      return (const gchar *)s;
}


const gchar *
sizetag(off_t tama, gint count)
{
    gchar *tag = "B";
    static gchar *buf=NULL;
    long long utama=(long long)tama;

    g_free(buf);buf=NULL;
    if(utama >= 0)
    {
	if(utama >= (long long)1024 * 1024 * 1024)
	{
	    utama /= ((long long)1024 *1024 * 1024);
	    tag = "GB";
	}
	else if(utama >= 1024 * 1024)
	{
	    utama /= (1024 * 1024);
	    tag = "MB";
	}
	else if(utama >= 1024)
	{
	    utama /= 1024;
	    tag = "KB";
	}
	if(count <= 0){
	    /* format for size column of regular files */
	    buf=g_strdup_printf("%lld %s", utama, tag);
	}
	else if(count == 1){
	    /* format for size column of folders (one file) */
	    buf=g_strdup_printf(_("%d file, %lld %s."), count, utama, tag);
	}
	else {
	    /* format for size column of folders (more than one file) */
	    buf=g_strdup_printf(_("%d files, %lld %s."), count, utama, tag);
	}
    }
    else
    {
	if(count < 0){
	    buf=g_strdup_printf(" ");
	}
	else if(count == 1){
	    /* format for count of processed files (single file)*/
	    buf=g_strdup_printf(_("%d file"), count);
	} else {
	    /* format for count of processed files (multiple files)*/
	    buf=g_strdup_printf(_("%d files"), count);
	}
    }
    return (const gchar *)buf;
}


const char *tod(void){
	time_t t=time(NULL);
	return ctime(&t);
}


void count_selection(GtkTreeModel * treemodel, 
		GtkTreePath * treepath, 
		GtkTreeIter * iter, 
		gpointer data)
{
    tree_details->selectionOK++;
    return ;
}

treestuff_t *get_treestuff(GtkTreeView *treeview){
    int i;
    for (i=0;i<TREECOUNT;i++){
	if (treeview == tree_details->treestuff[i].treeview) 
	    return (tree_details->treestuff)+i;
    }
    g_warning("treestuff != NULL");
    return NULL;
}

gint get_tree_id(GtkTreeView *treeview){
    int i;
    for (i=0;i<TREECOUNT;i++){
      /*printf("DBG: treeviewcmp  0x%x==0x%x\n",tree_details->treestuff[i].treeview,treeview);*/
	if (treeview == tree_details->treestuff[i].treeview) 
	    return i;
    }
    /*g_error("treestuff[tree_id] != NULL");*/
    return -1;
}


static gboolean correct_selection;
static void query_selection(GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
{
    /*GtkTreeView *treeview = (GtkTreeView *) data;*/
    if(correct_selection) return;
    correct_selection=TRUE;
    return;
}

 GtkTreeView *get_bigger_treeview(void){
    GtkWidget *vpaned = WIDGET("hpaned1");
    if (gtk_paned_get_position((GtkPaned *)vpaned) <= vpaned->allocation.width * 0.50) {
	return tree_details->treestuff[0].treeview;
    } else return tree_details->treestuff[1].treeview;
}

treestuff_t *get_only_visible_treestuff(void){
    GtkWidget *vpaned = WIDGET("hpaned1");
    if (gtk_paned_get_position((GtkPaned *)vpaned) <= vpaned->allocation.width * 0.10) {
	set_relative_tree_id(0);
	return tree_details->treestuff;
    } 
    if (gtk_paned_get_position((GtkPaned *)vpaned) >= vpaned->allocation.width *0.90) {
	set_relative_tree_id(1);
	return tree_details->treestuff+1;
    }
    return NULL;
}

gboolean local_branch_is_visible(GtkTreeModel *treemodel){
    GtkTreeIter iter;
    tree_entry_t *en;
    if(gtk_tree_model_get_iter_first(treemodel, &iter)){
	do {  
	  gtk_tree_model_get(treemodel, &iter, ENTRY_COLUMN, &en, -1);
	  if (!en) continue;
	  if (IS_LOCAL_TYPE(en->type)) return TRUE;
	} while (gtk_tree_model_iter_next(treemodel, &iter));
    }
    return FALSE;	
}


static GtkTreeView *get_active_treeview(void){
    int i;
    treestuff_t *treestuff=get_only_visible_treestuff();
    GtkTreeView *treeview;
    GtkTreeSelection *selection;
    correct_selection=FALSE;
    
    
    if (treestuff) return (treestuff->treeview);
    for (i=0;i<TREECOUNT;i++) {
      treeview=tree_details->treestuff[i].treeview;
      selection = gtk_tree_view_get_selection(treeview);
      gtk_tree_selection_selected_foreach(selection, query_selection, (gpointer) treeview);
      if (correct_selection) return treeview;
    }
    return get_bigger_treeview();
}

gint get_active_tree_id(void){
    gint i;
    GtkTreeView *treeview = get_active_treeview();
    i=get_tree_id(treeview);
    if (i < 0 || i >= TREECOUNT) 
	g_error("treeview != any(tree_details->treestuff[i].treeview)");
    return i;
}

/* new treeview selection algorithm */

static gint relative_one=0;

gint get_ascending(void){
    return tree_details->treestuff[relative_one].ascending;
}

gint get_relative_tree_id(void){
    return relative_one;
}
void set_relative_tree_id(gint i){
    relative_one=i;
}

gint get_relative_sort_column(void){
    return tree_details->treestuff[relative_one].sort_column;
}

void on_treeview_column_click(GtkTreeViewColumn * column, gpointer data){
    set_relative_tree_id((gint)((long)data));
}

gint set_relative_tree_id_from_model(GtkTreeModel *treemodel){
    int i;
    for (i=0;i<TREECOUNT;i++){
      /*printf("DBG: treeviewcmp  0x%x==0x%x\n",tree_details->treestuff[i].treeview,treeview);*/
	if (treemodel == tree_details->treestuff[i].treemodel) {
	    set_relative_tree_id(i);
	    return 1;
	}
    }
    g_warning("tree_id == NULL");
    return 1;
}

    

/* this function is to workaround a gtk bug in treeviews, where a 
 * while(gtk_events_pending()) will not break... 
 * Let's say 256 iterations suffice... 
 * But not always (if called from a timeout function),
 * so let's provide a blocking mechanism */
static gboolean pending_busy=FALSE;
gboolean processing_pending(void){
	return pending_busy;
}

void set_processing_pending(void){
	pending_busy=TRUE;
}
void unset_processing_pending(void){
	pending_busy=FALSE;
}

void process_pending_gtk(void){
  int i;
  static gboolean recursive=FALSE;
  /* breakout if recursive call */
  if (recursive) return;
  if (processing_pending()) return;
  recursive=TRUE;
  for (i=0;i<256;i++){
	  if (!gtk_events_pending()) break;
	  else gtk_main_iteration();
  }
  recursive=FALSE;  
}

void disable_diagnostics(void){diagnostics_disabled=TRUE;};
void enable_diagnostics(void){diagnostics_disabled=FALSE;};

const gchar *get_xffm_home(void){
    static gchar *home=NULL;
    g_free(home); 
    home=NULL;    
    if (getenv("XFFM_HOME") && strlen( getenv("XFFM_HOME"))){
	    home=g_strdup(getenv("XFFM_HOME"));
    }
    if (!home) home=g_strdup(GETWD);
    if (!g_file_test(home,G_FILE_TEST_IS_DIR) || access(home,X_OK)!=0) {
	g_free(home);
	home=g_strdup("/");
    }
    return home;
}

/* this could also delete tmp files in use by another xffm instance 
 * (very slight possibility)*/
void cleanup_tmpfiles(void)
{
    DIR *directory;
    char *fullpath;
    struct dirent *d;
    const char *path=g_get_tmp_dir();
    directory = opendir(path);
    if(!directory) return;
    while((d = readdir(directory)) != NULL)
    {
	if(strncmp(d->d_name, "xffm",strlen("xffm"))==0){
	   fullpath = g_build_filename((gchar *)path, d->d_name,NULL);
	   unlink(fullpath);
	   rmdir(fullpath);
	   g_free(fullpath);
	}
    }
    closedir(directory);
    return;
}

void fork_function(void *data)
{
    char **argument = (char **)data;

    execvp(argument[0], argument);
    fprintf(stderr, "CHILD could not execvp: this should not happen");
    fprintf(stderr, "Do you have %s in your path?", argument[0]);
    fflush(NULL);
    usleep(500000);
    _exit(123);
}

gint 
count_files(gchar *path)
{
    DIR *directory;
    struct dirent *dir;
    gint count = 0;

    directory = opendir(path);
    if(!directory)
	return -1;
    while((dir = readdir(directory)) != NULL)
    {
	if(strcmp(dir->d_name, ".") && strcmp(dir->d_name, ".."))		
		count++;
    }
    closedir(directory);
    return (count);
}

/* This function (count_hidden_files) is only used here... **/
static gint 
count_hidden_files(gchar *path)
{
    DIR *directory;
    struct dirent *dir;
    gint count = 0;

    directory = opendir(path);
    if(!directory) return -1;
    while((dir = readdir(directory)) != NULL)
    {
	if(strcmp(dir->d_name, ".") == 0 || 
	   strcmp(dir->d_name, "..") == 0 || 
	   strcmp(dir->d_name, "..Wastebasket") == 0)
	    continue;
	if(dir->d_name[0] == '.') count++;
    }
    closedir(directory);
    return (count);
}

void set_entry_tag(GtkTreeView * treeview, tree_entry_t * en, off_t tama) 
{
    int hcount;

    hcount = count_hidden_files(en->path);
    if(hcount)
    {
	SET_HAS_HIDDEN(en->type);
    }
    if (en->tag) g_free(en->tag);
    if(SHOWS_HIDDEN(en->type))
	en->tag = g_strdup_printf("%s (%s %s)", FILENAME(en), sizetag(tama, en->count), _("Showing hidden."));
    else
    {
	if(hcount)
	    en->tag = g_strdup_printf("%s (%s %d %s)", FILENAME(en), sizetag(tama, en->count), hcount, _("hidden."));

	else
	    en->tag = g_strdup_printf("%s (%s %s)", FILENAME(en), sizetag(tama, en->count), _("No hidden."));
    }
}

 
gboolean get_selectpath_iter(GtkTreeIter * iter, tree_entry_t ** en)
{
   gint tree_id = get_active_tree_id();
    GtkTreeModel *treemodel = tree_details->treestuff[tree_id].treemodel;
    GtkTreeSelection *selection = tree_details->treestuff[tree_id].selection;
    GtkTreeIter parent;
    tree_entry_t *c_en, *p_en = NULL;
    int caso = 0;

    if(!tree_details) return FALSE;
    if(!tree_details->window) return FALSE;


    gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
    if(!gtk_tree_selection_get_selected(selection, &treemodel, iter))
    {
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
	return FALSE;
    }
    gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);

    gtk_tree_model_get(treemodel, iter, ENTRY_COLUMN, &c_en, -1);
    if(gtk_tree_model_iter_parent(treemodel, &parent, iter))
    {
	gtk_tree_model_get(treemodel, &parent, ENTRY_COLUMN, &p_en, -1);
    }

    if(IS_PATH(c_en->type) && IS_DIR(c_en->type))
	caso = 1;

    else if(IS_FIND_TYPE(c_en->type))
    {
	if(IS_DIR(c_en->type))
	    caso = 1;
	else if(p_en && IS_FILE(c_en->type) && IS_XF_FND(p_en->type))
	    caso = 0;
	else if(p_en && IS_FILE(c_en->type) && IS_DIR(p_en->type))
	    caso = 2;
	else
	    caso = 0;
    }
    else if(IS_LOCAL_TYPE(c_en->type))
    {
	if(IS_DIR(c_en->type))
	    caso = 1;
	else if(p_en && (IS_FILE(c_en->type) || IS_DUMMY_TYPE(c_en->type)))
	    caso = 2;
	else
	    caso = 0;
    }
    else if(IS_BOOKMARK_TYPE(c_en->type))
    {
	if(IS_DIR(c_en->type) || IS_ROOT_TYPE(c_en->type) ||
	   IS_NETDIR(c_en->subtype) || IS_XF_NETSHARE(c_en->subtype) ||
	   IS_XF_NETWS(c_en->subtype)  )
	    caso = 1;
	else if (p_en && 
		 (IS_FILE(c_en->type) || IS_DUMMY_TYPE(c_en->type)
		  || IS_NETFILE(c_en->subtype))
		) caso = 2;
	else
	    caso = 0;
    }
    else if(IS_TRASH_TYPE(c_en->type) || IS_RECENT_TYPE(c_en->type) 
	    || IS_FREQUENT_TYPE(c_en->type))
    {
	if(IS_ROOT_TYPE(c_en->type))
	    caso = 1;
	else if(p_en && (IS_FILE(c_en->type) || IS_DUMMY_TYPE(c_en->type)))
	    caso = 2;
	else
	    caso = 0;
    }
    else if(IS_NETWORK_TYPE(c_en->type))
    {
	if(IS_ROOT_TYPE(c_en->type) || 
	   IS_XF_NETWG(c_en->subtype) ||
	   IS_XF_NETWS(c_en->subtype) ||
	   IS_NETDIR(c_en->subtype) ||
	   IS_XF_NETSHARE(c_en->subtype))
	    caso = 1;
	else if (p_en )
	    caso = 2;
	else
	    caso = 0;
    }
    else if (IS_FSTAB_TYPE(c_en->type))
    {
	if(IS_ROOT_TYPE(c_en->type)||IS_DIR(c_en->type))
	    caso = 1;
	else if(p_en && (IS_FILE(c_en->type) || IS_DUMMY_TYPE(c_en->type)))
	    		caso = 2;
	else caso = 0;
    }
	    
    /*printf("DBG: caso is %d\n",caso);*/
    if(caso)
    {
	if(caso == 2)
	{
	    *en = p_en;
	    *iter = parent;
	}
	else
	    *en = c_en;
    }
    else
	return FALSE;

    return TRUE;
}

int get_the_root(GtkTreeView * treeview, GtkTreeIter * iter, tree_entry_t ** en, int which)
{
    GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
    if(!gtk_tree_model_get_iter_first(treemodel, iter)){
new_root:
#ifdef DEBUG
	   printf("DBG(misc.c):create_root_element(treeview,iter,which,g)\n");
#endif
	   create_root_element(treeview,iter,which,NULL);
   	   gtk_tree_model_get(treemodel, iter, ENTRY_COLUMN, en, -1);
	   return TRUE;
    }
	   
	    
    gtk_tree_model_get(treemodel, iter, ENTRY_COLUMN, en, -1);

    while((((*en)->type) & __TREE_MASK) != root[which].type)
    {
	if(!gtk_tree_model_iter_next(treemodel, iter))
	   goto new_root;
	gtk_tree_model_get(treemodel, iter, ENTRY_COLUMN, en, -1);
    }
    return TRUE;
}

gboolean find_root(GtkTreeView * treeview, int which)
{
    GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
    GtkTreeIter iter;
    tree_entry_t *en;
    if(!gtk_tree_model_get_iter_first(treemodel, &iter)) return FALSE;
	   
	    
    gtk_tree_model_get(treemodel, &iter, ENTRY_COLUMN, &en, -1);

    while(((en->type) & __TREE_MASK) != root[which].type)
    {
	if(!gtk_tree_model_iter_next(treemodel, &iter))
	   return FALSE;
	gtk_tree_model_get(treemodel, &iter, ENTRY_COLUMN, &en, -1);
    }
    return TRUE;
}

int erase_the_root(GtkTreeView * treeview, int which)
{
    GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
    GtkTreeIter target;
    tree_entry_t *en;

    if(!gtk_tree_model_get_iter_first(treemodel, &target)){
	    return FALSE;
    } 
    gtk_tree_model_get(treemodel, &target, ENTRY_COLUMN, &en, -1);
    if (!en) return FALSE;

    while(((en->type) & __TREE_MASK) != root[which].type)
    {
	if(!gtk_tree_model_iter_next(treemodel, &target)){
	   return FALSE;
	}
	gtk_tree_model_get(treemodel, &target, ENTRY_COLUMN, &en, -1);
	if (!en) return FALSE;
    }
    remove_row(treemodel, &target, NULL, en);
    return TRUE;
}

static const gchar *abbreviate(char *string,int length){
    int i;
    static gchar *label = NULL;

    if (!string) return "";
    if(label) g_free(label);
    label = g_strdup(string);
    
    if(strlen(label) <= length) return my_utf_string((const gchar *)label);

    label[0] = '~';
    for(i = 1; i <= length; i++)
	label[i] = label[strlen(label) - (length - i)];
    return label;
}

const gchar *abbreviate_end(gchar *string,int length){
    static gchar *label = NULL;

    if (!string) return "";
    if(label) g_free(label);
    label = g_strdup(string);
    
    if(strlen(label) <= length) return my_utf_string((const gchar *)label);

    label[length-1] = '~';
    label[length] = 0;
    return label;
}

const char *abreviate(gchar *string)
{
     return abbreviate(string,35);
}


static gchar * my_cut_utf_string(gchar *s){
	static  gchar *u=NULL;
	gchar *v=g_strdup(my_utf_string(s));
	int max_len=48;
	if (getenv("XFFM_STATUS_LINE_LENGTH") && strlen( getenv("XFFM_STATUS_LINE_LENGTH"))) {
		if (is_number(getenv("XFFM_STATUS_LINE_LENGTH")))
		   max_len=(atoi(getenv("XFFM_STATUS_LINE_LENGTH")));
	}
	if (u) {
		g_free(u);
		u=NULL;
	}
        if (g_utf8_strlen(s,-1) > max_len){
	    u=g_utf8_offset_to_pointer (v,max_len-4);
	    *u=0;
	    u=g_strjoin ("",v,"...",NULL);
	    g_free(v);
	    v=NULL;
       } else u=v;
       return u;
} 

gchar *randomTmpName(char *ext)
{
    gchar *fname;
    gchar *rname;
    rname = g_strconcat(g_get_tmp_dir(),G_DIR_SEPARATOR_S,"xffm.XXXXXX",NULL);
    D(printf("DBG: rname=%s\n",(rname)?rname:"null");)
    close(mkstemp(rname));
    if(ext == NULL) fname = g_strdup(rname);
    else fname = g_strconcat(rname,".",ext,NULL);
    D(printf("DBG: fname=%s\n",(fname)?fname:"null");)
    g_free(rname);
    return fname;
}

const gchar *xffm_filename(gchar *path)
{
    gchar *name=NULL;
    
    g_free (name);

    if (!path) return "";
    name = g_path_get_basename(path);
    if (!name) return "";
    return (const gchar *)name;
}


static const gchar *plain_sizetag(int tama, int tamal)
{
    char *tag = "KB";
    char *tagl = "KB";
    static gchar *buf=NULL;

    g_free(buf);
    buf=NULL;
    if(tama >= 1024 * 1024)
    {
	tama /= (1024 *1024);
	tag = "GB";
    }
    else if(tama >= 1024)
    {
	tama /= 1024;
	tag = "MB";
    }

    if(tamal >= 1024 * 1024)
    {
	tamal /= (1024 *1024);
	tagl = "GB";
    }
    else if(tamal >= 1024)
    {
	tamal /= 1024;
	tagl = "MB";
    }

    buf=g_strdup_printf("%d %s / %d %s", tama, tag, tamal, tagl);

    return (const gchar *)buf;
}


void set_progress_generic(int count, int total, int caso)
{
    static int smallcount = 1;

    gfloat fraction;

    GtkWidget *progress;

    if(!tree_details || !tree_details->window)
	return;
    progress = WIDGET("progressbar1");
    if(!progress)
	return;
    if(count == -1)
    {
	/*printf("DBG:updating pulse\n"); */
	if(total < 0 || (smallcount++ & (1 << 7)))
	{
	    gtk_progress_bar_pulse((GtkProgressBar *) progress);
    process_pending_gtk();
	    smallcount = 1;
	}
    }
    else
    {
	char texto[_POSIX_PATH_MAX];
	if(!total)
	    fraction = 0.0;
	else
	    fraction = (float)count / (float)total;
	if(fraction < 0 || fraction > 1)
	    return;
	gtk_progress_bar_set_fraction((GtkProgressBar *) progress, fraction);
	if(count >= total)
	    texto[0] = 0;
	else
	    switch (caso)
	    {
		case 1:
		    sprintf(texto, "%d / %d", count, total);
		    break;
		case 0:
		default:
		    sprintf(texto, "%s", plain_sizetag(count, total));
		    break;
	    }
	gtk_progress_bar_set_text((GtkProgressBar *) progress, texto);
    }
}


void cursor_wait(void)
{
    static GdkCursor *cursor = NULL;
    if(!tree_details->window)
	return;
    if(!cursor)
	cursor = gdk_cursor_new(GDK_WATCH);
    gdk_window_set_cursor(tree_details->window->window, cursor);
    /*animation(treeview,TRUE); */
    gdk_flush();
}

void cursor_reset(void)
{
    if(!tree_details->window)
	return;
    gdk_window_set_cursor(tree_details->window->window, NULL);
    /*animation(treeview,FALSE); */

}

void set_font(GtkTreeModel *treemodel, GtkTreeIter * iterator)
{
    PangoFontDescription *desc;
    int size,ssize;
    tree_entry_t *en;
   
    if (!(tree_details->preferences & ENABLE_RESIZE_FONTS)) return;
       gtk_tree_model_get(treemodel, iterator, ENTRY_COLUMN, &en, -1);
    if (!en) return;   
    switch (tree_details->icon_size) {
	case 3: 
	    size = 16 * 1000;break;
	case 2:
	    size = 14 * 1000;break;
	case 1:
	    size = 12 * 1000;break;
	case 0:
	default:
	    size = 10 * 1000;break;
    }
    ssize = 10 * 1000;
    desc = pango_font_description_new();

#if 0
    if(IS_ROOT_TYPE(en->type))
	pango_font_description_set_family(desc, "times");
#endif

    pango_font_description_set_size(desc, size);

    /*printf("font size is=%d\n",pango_font_description_get_size(desc)); */
//FIXME: all gtk_tree_store_set should reside in "basic" files
    gtk_tree_store_set((GtkTreeStore *) treemodel, iterator, FONT_COLUMN, desc, -1);
    pango_font_description_set_size(desc, ssize);
    gtk_tree_store_set((GtkTreeStore *) treemodel, iterator, SFONT_COLUMN, desc, -1);
    
    pango_font_description_free(desc);
}


void hideit(GtkWidget * parent, const gchar * widget_name)
{
    GtkWidget *widget;
    if(!parent)
	return;
    if((widget = lookup_widget(parent,widget_name)) != NULL)
    {
	if(GTK_WIDGET_VISIBLE(widget))
	    gtk_widget_hide_all(widget);
    }

}
void showit(GtkWidget * parent, const gchar * widget_name)
{
    GtkWidget *widget;
    if(!parent)
	return;
    if((widget = lookup_widget(parent,widget_name)) != NULL)
    {
	if(!GTK_WIDGET_VISIBLE(widget))
	    gtk_widget_show_all(widget);
    }

}

void show_text(GtkWidget * parent)
{
    GtkWidget *vpaned;
    if(!parent)
	return;
    /*printf("DBG:show_text\n");*/
    vpaned = WIDGET("vpaned1");
    if (gtk_paned_get_position((GtkPaned *)vpaned) > vpaned->allocation.height * 0.80)
       gtk_paned_set_position(GTK_PANED(vpaned), vpaned->allocation.height * 0.80);
}

void hide_text(GtkWidget * parent)
{
    GtkWidget *vpaned;
    vpaned = WIDGET("vpaned1");
    gtk_paned_set_position(GTK_PANED(vpaned), vpaned->allocation.height);
}

const gchar *utf_2_local_string(const gchar *t){
    static gchar *s = NULL;
    GError *error = NULL;
    gsize r_bytes, w_bytes;
    const char *fc;
    gchar *from_codeset=NULL;

    
    g_free(s);
    s=NULL;
    g_get_charset(&fc);
    if (fc) from_codeset = g_strdup(fc);
    else from_codeset = g_strdup("ISO-8859-1");
    
    
    if (strcmp(from_codeset,"ASCII")==0){
	    g_free(from_codeset);
	    from_codeset = g_strdup("ISO-8859-1");
    }    
#ifdef DEBUG    
    printf("DBG: codeset for system is: %s\n",from_codeset);    
#endif
    
#ifdef DEBUG    
    printf("DBG: setting to local codeset-> %s\n",t); 
#endif
    if (strcmp(from_codeset,"UTF-8")==0) s = (gchar *)t;
    else {
	s = g_convert (t,strlen(t),from_codeset,"UTF-8",&r_bytes, &w_bytes, &error);
	if(error){
	    print_diagnostics("xfce/error_icon",error->message,"\n",NULL);
	    g_error_free(error);
	}
    }
    g_free(from_codeset);
    return (const gchar *)s;
}

const gchar *my_utf_string(const gchar *t)
{
    static gchar *s = NULL;
    GError *error = NULL;
    gsize r_bytes, w_bytes;
    unsigned char *c;
    const char *fc;
    gchar *from_codeset=NULL;
    
    if(!t) {
	g_warning("my_utf_string(NULL)");
	return "";
    }
    
    if (g_utf8_validate (t,-1,NULL)) {
#ifdef DEBUG    
    /*printf("DBG: valid utf8: %s\n",t);    */
#endif
	    return t;
    }
    
    /* so we got a non-UTF-8 */
    
    if (getenv("SMB_CODESET") && strlen( getenv("SMB_CODESET"))){
	    from_codeset=g_strdup(getenv("SMB_CODESET"));
    }
    else {
    	g_get_charset(&fc);
    	if (fc) from_codeset = g_strdup(fc);
    	else from_codeset = g_strdup("ISO-8859-1");
    }
    
/* hey, rsh will give you ANSI_X3.4-1968, so let's limit conversion with ISO */
/*    if (strcmp(from_codeset,"UTF-8")==0 || strcmp(from_codeset,"ASCII")==0){*/
    if (!strcmp(from_codeset,"ISO-")){
	    g_free(from_codeset);
	    from_codeset = g_strdup("ISO-8859-1");
    }    

#ifdef DEBUG    
    printf("DBG: codeset for system is: %s\n",from_codeset);    
#endif
    
    if(s) {
	    g_free(s);
	    s=NULL;
    }
#ifdef DEBUG    
    printf("DBG: utfing %s\n",t); 
#endif
    for(c = (unsigned char *)t; *c != 0; c++)
	if(*c < 32 && *c != '\n')
	    *c = ' ';
    s = g_convert (t,strlen(t),"UTF-8",from_codeset,&r_bytes, &w_bytes, &error);
    g_free(from_codeset);
    from_codeset=NULL;

    if(!s)
    {
	s=g_strdup(t);
	for(c = s; *c != 0; c++) if(*c > 128) *c = '?';
    }
    if(error){
        printf("DBG: %s. Codeset for system is: %s\n",
			error->message,from_codeset);
        printf("DBG: You should set the environment variable SMB_CODESET to ISO-8859-1\n");
	g_error_free(error);
    }
    return (const gchar *)s;
}


/*#define PANGO_IS_BUGGED*/

static void insert_string(GtkTextBuffer * buffer, const gchar *s)
{
    GtkTextIter start,end;
    if(s)
    {
#ifdef PANGO_IS_BUGGED
	gchar *o=NULL,*g=NULL;
	gtk_text_buffer_get_bounds(buffer,&start, &end);
	o=gtk_text_buffer_get_text(buffer,&start,&end,TRUE);
	g=g_strconcat(o,s,NULL);

	gtk_text_buffer_set_text  (buffer,(const gchar *)g,-1);
	g_free(o);
	g_free(g);
#else 
	gtk_text_buffer_get_bounds(buffer,&start, &end);
	gtk_text_buffer_insert(buffer, &end, s, -1);
#endif
    }
    return;
}

gboolean is_number(char *p){
   char *c=p;
   if (!c || !strlen(c)) return FALSE;
   for (;*c;c++) {
	if (*c < '0' || *c > '9') return FALSE;
   }
   return TRUE;   
}


void print_status_tmp(GdkPixbuf * icono, ...)
{
    va_list var_args;
    char *t;
    GtkTextIter start, end;
    GtkTextBuffer *buffer = gtk_text_view_get_buffer((GtkTextView *)WIDGET("status"));

    if(!tree_details->window)
	return;
    gtk_text_buffer_get_bounds(buffer, &start, &end);
    gtk_text_buffer_delete(buffer, &start, &end);
    if(icono)
	gtk_text_buffer_insert_pixbuf(buffer, &end, icono);
    gtk_text_buffer_get_bounds(buffer, &start, &end);
    gtk_text_buffer_insert(buffer, &end, "  ", -1);
    va_start(var_args, icono);
    do
    {
	t = va_arg(var_args, char *);
	if(t && strlen(t)){
	    insert_string(buffer, my_cut_utf_string(t));
	}
    }
    while(t);
    va_end(var_args);
  gdk_flush();
}



void print_status(const gchar *id, ...)
{
    va_list var_args;
    char *t;
    GtkTextIter start, end;
    GtkTextBuffer *buffer = gtk_text_view_get_buffer((GtkTextView *)WIDGET("status"));

    if(!tree_details->window)
	return;
    gtk_text_buffer_set_text(buffer, " ", -1);
    gtk_text_buffer_get_bounds(buffer, &start, &end);
    if(id && icon_tell(0, id))
	gtk_text_buffer_insert_pixbuf(buffer, &end, icon_tell(0, id));
    gtk_text_buffer_get_bounds(buffer, &start, &end);
    gtk_text_buffer_insert(buffer, &end, "  ", -1);
    va_start(var_args, id);
    do
    {
	t = va_arg(var_args, char *);
	if(t && strlen(t)){	
	    insert_string(buffer, my_cut_utf_string(t));
	}
    }
    while(t);
    va_end(var_args);
  gdk_flush();
}


void print_diagnostics(const gchar *id, ...)
{
    char *t;
    va_list var_args;
    GtkTextIter start, end;
    GtkTextMark *mark;
    GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(WIDGET("diagnostics")));

    if(!tree_details->window || diagnostics_disabled)
	return;
    if (getenv("XFFM_DIAGNOSTICS_DISABLED") && strlen(getenv("XFFM_DIAGNOSTICS_DISABLED"))) return;
    if (!(tree_details->preferences & VERBOSE) && !id) return;
    if (id && strcmp(id,"nonverbose")==0) id=NULL;
    show_text((GtkWidget *)tree_details->window);
    /*printf("DBG:print_diagnostics\n");*/
    gtk_text_buffer_get_bounds(buffer, &start, &end);
    if(id && icon_tell(0, id))
	gtk_text_buffer_insert_pixbuf(buffer, &end, icon_tell(0, id));

    va_start(var_args, id);
    do
    {
	t = va_arg(var_args, char *);
	if(t && strlen(t)){
	    insert_string(buffer, my_utf_string(t));
	}
    }
    while(t);
    va_end(var_args);
    gtk_text_buffer_get_bounds(buffer, &start, &end);
    mark = gtk_text_buffer_create_mark(buffer, "scrollmark", &end, FALSE);
    gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(WIDGET("diagnostics")), mark, 0.2,	/*gdouble within_margin, */
				 FALSE, 0.0, 0.0);
    gtk_text_buffer_delete_mark(buffer, mark);
  gdk_flush();
}
void print_diagnostics_tmp(GdkPixbuf * icono, ...)
{
    char *t;
    va_list var_args;
    GtkTextIter start, end;
    GtkTextMark *mark;
    GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(WIDGET("diagnostics")));

    if(!tree_details->window || diagnostics_disabled)
	return;
    /*printf("DBG:print_diagnostics_tmp\n");*/
    show_text((GtkWidget *)tree_details->window);
    gtk_text_buffer_get_bounds(buffer, &start, &end);
    if(icono)
	gtk_text_buffer_insert_pixbuf(buffer, &end, icono);
 
    va_start(var_args, icono);
    do
    {
	t = va_arg(var_args, char *);
	if(t && strlen(t)){
	    insert_string(buffer, my_utf_string(t));
	}
    }
    while(t);
    va_end(var_args);

    gtk_text_buffer_get_bounds(buffer, &start, &end);
    mark = gtk_text_buffer_create_mark(buffer, "scrollmark", &end, FALSE);
    gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(WIDGET("diagnostics")), mark, 0.2,	/*gdouble within_margin, */
				 FALSE, 0.0, 0.0);
    gtk_text_buffer_delete_mark(buffer, mark);
  gdk_flush();
}


void clear_diagnostics(void)
{
    GtkTextIter start, end;
    GtkTextBuffer *buffer;
    if(!tree_details->window)
	return;
    buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(WIDGET("diagnostics")));
    gtk_text_buffer_get_bounds(buffer, &start, &end);
    gtk_text_buffer_delete(buffer, &start, &end);
    hide_text(tree_details->window);
}

void show_stop(void)
{
    hideit(tree_details->window, "clear_text");
    showit(tree_details->window, "stop");
    /*printf("DBG: showing stop\n"); */
    process_pending_gtk();
}

void hide_stop(void)
{
    hideit(tree_details->window, "stop");
    showit(tree_details->window, "clear_text");
    /*printf("DBG: hiding stop\n"); */
}

GtkTreeIter *
get_iter_from_reference(	GtkTreeModel * treemodel, 
				GtkTreeRowReference *reference)
{
    static GtkTreeIter iter;
    GtkTreePath *treepath;
    if (!gtk_tree_row_reference_valid(reference)){
        g_warning("Invalid row reference");
	return NULL;
    } else if ((treepath = gtk_tree_row_reference_get_path(reference))==NULL){
	g_warning("gtk_tree_row_reference_get_path() == NULL");
	return NULL;
    }
    gtk_tree_model_get_iter(treemodel, &iter, treepath);
    gtk_tree_path_free(treepath);
    return &iter;
}



