/*
 * Copyright (C) 2003 Edscott Wilson Garcia
 * EMail: edscott@imp.mx
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#define __MODULES_C__
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <gmodule.h>

#include "modules.h"

#ifndef DEBUG
#define DEBUG
#endif

G_MODULE_IMPORT
void print_diagnostics(char *id, ...);
G_MODULE_IMPORT
int runvwd(const gchar *workdir,char *argv[]);

G_MODULE_IMPORT
int extra_key_completion(gpointer extra_key_data);

static GModule *xfc_cm=NULL;
xfc_combo_functions *load_xfc(void){
    xfc_combo_functions *(*module_init)(void) ;
    gchar *library, *module;
    static xfc_combo_functions *xfc_fun=NULL;
   
    if (xfc_fun) return xfc_fun;
    
    library=g_strconcat("libxfce4_combo.",G_MODULE_SUFFIX, NULL);
    module = g_build_filename (LIBDIR, "xfce4","modules",library, NULL);
    
    xfc_cm=g_module_open (module, 0);
    if (!xfc_cm){
	g_error("g_module_open(%s) == NULL\n",module);
   	exit(1);
    }
    
    if (!g_module_symbol (xfc_cm, "module_init",(gpointer) &(module_init)) ) {
	g_error("g_module_symbol(module_init) != FALSE\n");
        exit(1);
    }

    
    xfc_fun = (*module_init)();
    xfc_fun->extra_key_completion = extra_key_completion;
    xfc_fun->extra_key_data = NULL;
 
#ifdef DEBUG
    g_message ("xffm: module %s successfully loaded", library);	    
#endif
    g_free(library);
    g_free(module);
    return xfc_fun;
}

void unload_xfc(void){
	if (!g_module_close(xfc_cm)){
	   g_warning("g_module_close(xfc_cm) != TRUE\n");
	}
#ifdef DEBUG
	else {
           g_message ("xffm: module libxffm_combo unloaded");
	}	   
#endif
	xfc_cm=NULL;
}

static xfprop_functions *xfprop_fun=NULL;
static GModule *xfprop_cm=NULL;
void unload_prop_module(void){
	g_free(xfprop_fun);
	xfprop_fun = NULL;
	if (!g_module_close(xfprop_cm)){
	   g_warning("g_module_close(xfprop_cm) != TRUE\n");
	}
#ifdef DEBUG
	else {
           g_message ("xffm: module libxffm_prop unloaded");
	}	   
#endif
	xfprop_cm=NULL;
}

xfprop_functions *load_prop_module(void){
    gchar *library, *module;
    xfprop_functions *(*module_init)(void) ;
    
    if (xfprop_fun) return xfprop_fun;
    
    library=g_strconcat("libxffm_prop.",G_MODULE_SUFFIX, NULL);
    module = g_build_filename (LIBDIR, "xffm", library, NULL);
    
    xfprop_cm=g_module_open (module, 0);
    if (!xfprop_cm){
	g_error("g_module_open(%s) == NULL\n",module);
   	exit(1);
    }
   if (!g_module_symbol (xfprop_cm, "module_init",(gpointer) &(module_init)) ) {
	g_error("g_module_symbol(module_init) != FALSE\n");
        exit(1);
    }

    
    xfprop_fun = (*module_init)();
    /* this function is exported to the module */
    xfprop_fun->print_diagnostics = print_diagnostics;
    xfprop_fun->runvwd = runvwd;
       
    /* this function is imported from the module */
    if (!g_module_symbol (xfprop_cm, "do_prop",(gpointer) &(xfprop_fun->do_prop)) ) {
	g_error("g_module_symbol(do_prop) != FALSE\n");
        exit(1);
    }
#ifdef DEBUG
    g_message ("xffm: module %s successfully loaded", library);	    
#endif
    g_free(library);
    g_free(module);
    return xfprop_fun;
}


static xfmime_icon_functions *xfmime_icon_fun=NULL;
static GModule *xfmime_icon_cm=NULL;
void unload_mime_icon_module(void){
	if (!xfmime_icon_fun) return;
	g_free(xfmime_icon_fun);
	xfmime_icon_fun = NULL;
	

	if (!g_module_close(xfmime_icon_cm)){
	   g_warning("g_module_close(xfmime_icon_cm) != TRUE\n");
	}
#ifdef DEBUG
	else {
           g_message ("xffm: module libxfce4_mime_icons unloaded");
	}	   
#endif
	xfmime_icon_cm=NULL;
}

xfmime_icon_functions *load_mime_icon_module(void){
    gchar *library, *module;
    xfmime_icon_functions *(*module_init)(void) ;
    
    if (xfmime_icon_fun) return xfmime_icon_fun;
    
    library=g_strconcat("libxfce4_mime_icons.",G_MODULE_SUFFIX, NULL);
    module = g_build_filename (LIBDIR, "xfce4","modules", library, NULL);
    
    xfmime_icon_cm=g_module_open (module, 0);
    if (!xfmime_icon_cm){
	g_error("g_module_open(%s) == NULL\n",module);
   	exit(1);
    }
    
   if (!g_module_symbol (xfmime_icon_cm, "module_init",(gpointer) &(module_init)) ) {
	g_error("g_module_symbol(module_init) != FALSE\n");
        exit(1);
    }

    
    xfmime_icon_fun = (*module_init)();
    
#ifdef DEBUG
    g_message ("xffm: module %s successfully loaded", library);	    
#endif
    g_free(library);
    g_free(module);
    return xfmime_icon_fun;
}


static xfmime_functions *xfmime_fun=NULL;
static GModule *xfmime_cm=NULL;
void unload_mime_module(void){
	if (!xfmime_fun) return;
	g_free(xfmime_fun);
	xfmime_fun = NULL;

	if (!g_module_close(xfmime_cm)){
	   g_warning("g_module_close(xfmime_cm) != TRUE\n");
	}
#ifdef DEBUG
	else {
           g_message ("xffm: module libxfce4_mime unloaded");
	}	   
#endif
	xfmime_cm=NULL;
}

xfmime_functions *load_mime_module(void){
    gchar *library, *module;
    xfmime_functions *(*module_init)(void) ;
    
    if (xfmime_fun) return xfmime_fun;
    
    library=g_strconcat("libxfce4_mime.",G_MODULE_SUFFIX, NULL);
    module = g_build_filename (LIBDIR, "xfce4","modules", library, NULL);
    
    xfmime_cm=g_module_open (module, 0);
    if (!xfmime_cm){
	g_error("g_module_open(%s) == NULL\n",module);
   	exit(1);
    }
   if (!g_module_symbol (xfmime_cm, "module_init",(gpointer) &(module_init)) ) {
	g_error("g_module_symbol(module_init) != FALSE\n");
        exit(1);
    }

    
    xfmime_fun = (*module_init)();
#ifdef DEBUG
    g_message ("xffm: module %s successfully loaded", library);	    
#endif
    g_free(library);
    g_free(module);
    return xfmime_fun;
}



#if 0
gpointer load_do_find(void){
    static GModule *do_find_cm=NULL;
    static gpointer do_find=NULL;
    gchar *library, *module;
   
    if (do_find) return do_find;
    
    library=g_strconcat("libxffm_fgr.",G_MODULE_SUFFIX, NULL);
    module = g_build_filename (LIBDIR, "xffm", library, NULL);
    
    do_find_cm=g_module_open (module, 0);
    if (!do_find_cm){
	g_error("g_module_open(%s) == NULL\n",module);
   	exit(1);
    }
       
    if ( !g_module_symbol (do_find_cm, "do_find", &do_find) ) {
	g_error("g_module_symbol(do_find) != FALSE\n");
        exit(1);
    }
#ifdef DEBUG
    g_message ("xffm: module %s successfully loaded", library);	    
#endif
    g_free(library);
    g_free(module);
    return do_find;
}
#endif

